import typing
from typing import Optional, cast

import requests
from requests import Session
from typing_extensions import Self

if typing.TYPE_CHECKING:
    from yayarl import URL


class RequestsMixin:
    # should only be used in URL!!!
    _default_session: Session | None = None

    def _get_session(self, session: Optional[Session] | None) -> Session:
        return cast(Session,
                    session or self._default_session or requests
                    )

    def options(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).options(str(self), **kwargs)

    def head(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).head(str(self), **kwargs)

    def get(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).get(str(self), **kwargs)

    def post(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).post(str(self), **kwargs)

    def patch(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).patch(str(self), **kwargs)

    def put(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).put(str(self), **kwargs)

    def delete(self, session: Optional[Session] = None, **kwargs):
        return self._get_session(session).delete(str(self), **kwargs)

    def __and__(self, other: Optional[Session]) -> "URL":
        cls = cast(typing.Type["URL"], self.__class__)
        new = cls(str(self))  # create a copy
        new._default_session = other
        return new



import os
import pathlib
import re
import sys

from setuptools import Extension, setup

NO_EXTENSIONS = bool(os.environ.get("YARL_NO_EXTENSIONS"))  # type: bool

if sys.implementation.name != "cpython":
    NO_EXTENSIONS = True


extensions = [
    # Extension("yarl._quoting_c", ["yayarl/_quoting_c.c"])
]


here = pathlib.Path(__file__).parent


def read(name):
    fname = here / name
    with fname.open(encoding="utf8") as f:
        return f.read()


# $ echo ':something:`test <sdf>`' | sed 's/:\w\+:`\(\w\+\)\(\s\+\(.*\)\)\?`/``\1``/g'
# ``test``
def sanitize_rst_roles(rst_source_text: str) -> str:
    """Replace RST roles with inline highlighting."""
    role_regex = r":\w+:`(?P<rendered_text>[^`]+)(\s+(.*))?`"
    substitution_pattern = r"``(?P=rendered_text)``"
    return re.sub(role_regex, substitution_pattern, rst_source_text)


args = dict(
    long_description="\n\n".join(
        [read("README.rst"), sanitize_rst_roles(read("CHANGES.rst"))]
    ),
)


if not NO_EXTENSIONS:
    print("**********************")
    print("* Accelerated build *")
    print("**********************")
    setup(ext_modules=extensions, **args)
else:
    print("*********************")
    print("* Pure Python build *")
    print("*********************")
    setup(**args)
